@@ -252,16 +252,17 @@ Pipe_func_connect(Pipe *self, PyObject *args)
 static PyObject *
 Pipe_func_open(Pipe *self, PyObject *args)
 {
-    int r, fd;
+    int r;
+    long fd;
 
     RAISE_IF_HANDLE_NOT_INITIALIZED(self, NULL);
     RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
 
-    if (!PyArg_ParseTuple(args, "i:open", &fd)) {
+    if (!PyArg_ParseTuple(args, "l:open", &fd)) {
         return NULL;
     }
 
-    r = uv_pipe_open(&self->pipe_h, fd);
+    r = uv_pipe_open(&self->pipe_h, (uv_file)fd);
     if (r != 0) {
         RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_PipeError);
         return NULL;
@@ -427,8 +428,7 @@ static int
 Pipe_tp_traverse(Pipe *self, visitproc visit, void *arg)
 {
     Py_VISIT(self->on_new_connection_cb);
-    StreamType.tp_traverse((PyObject *)self, visit, arg);
-    return 0;
+    return StreamType.tp_traverse((PyObject *)self, visit, arg);
 }
 
 
@@ -436,8 +436,7 @@ static int
 Pipe_tp_clear(Pipe *self)
 {
     Py_CLEAR(self->on_new_connection_cb);
-    StreamType.tp_clear((PyObject *)self);
-    return 0;
+    return StreamType.tp_clear((PyObject *)self);
 }
 
 
