@@ -242,11 +242,9 @@ callback:
     Py_DECREF(dns_result);
     Py_DECREF(errorno);
 
-    Py_DECREF(loop);
-    Py_DECREF(gai_req->callback);
-    Py_DECREF(gai_req);
-
     uv_freeaddrinfo(res);
+    UV_REQUEST(gai_req) = NULL;
+    Py_DECREF(gai_req);
 
     PyGILState_Release(gstate);
 }
@@ -266,10 +264,11 @@ Util_func_getaddrinfo(PyObject *obj, PyObject *args, PyObject *kwargs)
 
     UNUSED_ARG(obj);
     gai_req = NULL;
+    idna = NULL;
     port = socktype = protocol = flags = 0;
     family = AF_UNSPEC;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!OO|iiiii:getaddrinfo", kwlist, &LoopType, &loop, &host, &callback, &port, &family, &socktype, &protocol, &flags)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!OO|iiiii:getaddrinfo", kwlist, &LoopType, &loop, &callback, &host, &port, &family, &socktype, &protocol, &flags)) {
         return NULL;
     }
 
@@ -283,32 +282,27 @@ Util_func_getaddrinfo(PyObject *obj, PyObject *args, PyObject *kwargs)
     } else if (PyBytes_Check(host)) {
         host_str = PyBytes_AsString(host);
     } else {
-        PyErr_SetString(PyExc_TypeError, "getaddrinfo() argument 2 must be string or None");
-        return NULL;
+        PyErr_SetString(PyExc_TypeError, "getaddrinfo() argument 3 must be string or None");
+        goto error;
     }
 
     if (!PyCallable_Check(callback)) {
         PyErr_SetString(PyExc_TypeError, "a callable is required");
-        return NULL;
+        goto error;
     }
 
-    if (port < 0 || port > 65535) {
+    if (port < 0 || port > 0xffff) {
         PyErr_SetString(PyExc_ValueError, "port must be between 0 and 65535");
-        return NULL;
+        goto error;
     }
     snprintf(port_str, sizeof(port_str), "%d", port);
 
-    gai_req = (GAIRequest *)PyObject_CallObject((PyObject *)&GAIRequestType, NULL);
+    gai_req = (GAIRequest *)PyObject_CallFunctionObjArgs((PyObject *)&GAIRequestType, loop, callback, NULL);
     if (!gai_req) {
         PyErr_NoMemory();
-        return NULL;
+        goto error;
     }
 
-    gai_req->callback = callback;
-    REQUEST(gai_req)->loop = loop;
-    Py_INCREF(loop);
-    Py_INCREF(callback);
-
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = family;
     hints.ai_socktype = socktype;
@@ -321,13 +315,14 @@ Util_func_getaddrinfo(PyObject *obj, PyObject *args, PyObject *kwargs)
         goto error;
     }
 
+    Py_XDECREF(idna);
+
     Py_INCREF(gai_req);
     return (PyObject *)gai_req;
 
 error:
-    Py_DECREF(gai_req);
-    Py_DECREF(loop);
-    Py_DECREF(callback);
+    Py_XDECREF(idna);
+    Py_XDECREF(gai_req);
     return NULL;
 }
 
@@ -492,16 +487,14 @@ SignalChecker_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 static int
 SignalChecker_tp_traverse(SignalChecker *self, visitproc visit, void *arg)
 {
-    HandleType.tp_traverse((PyObject *)self, visit, arg);
-    return 0;
+    return HandleType.tp_traverse((PyObject *)self, visit, arg);
 }
 
 
 static int
 SignalChecker_tp_clear(SignalChecker *self)
 {
-    HandleType.tp_clear((PyObject *)self);
-    return 0;
+    return HandleType.tp_clear((PyObject *)self);
 }
 
 
