@@ -1,9 +1,9 @@
 
-import common
+from common import unittest2
 import pyuv
 
 
-class MultiHandleTest(common.UVTestCase):
+class MultiHandleTest(unittest2.TestCase):
 
     def test_multihandle1(self):
         self.close_cb_called = 0
@@ -46,7 +46,5 @@ class MultiHandleTest(common.UVTestCase):
 
 
 if __name__ == '__main__':
-    import unittest
-    tests = unittest.TestSuite(common.suites)
-    unittest.TextTestRunner(verbosity=2).run(tests)
+    unittest2.main(verbosity=2)
 
