@@ -1,5 +1,6 @@
 # coding=utf8
 
+import os
 import socket
 import sys
 
@@ -144,7 +145,9 @@ class TCPTest3(unittest2.TestCase):
         server.accept(connection)
         while connection.write_queue_size == 0:
             connection.write(b"PING"*1000)
-        connection.close()
+        connection.close(self.on_connection_close)
+
+    def on_connection_close(self, connection):
         self.client.close()
         self.server.close()
 
@@ -154,6 +157,9 @@ class TCPTest3(unittest2.TestCase):
         self.assertTrue(client.writable)
 
     def test_tcp_write_saturate(self):
+        if 'TRAVIS' in os.environ:
+            self.skipTest("Test disabled on Travis")
+            return
         self.server = pyuv.TCP(self.loop)
         self.server.bind(("0.0.0.0", TEST_PORT))
         self.server.listen(self.on_connection)
